package com.jiuqi.db;

import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.sql.ResultSet;
import java.sql.Statement;
import java.sql.Connection;
import java.util.Calendar;
import java.io.OutputStream;
import java.io.PrintStream;
import com.jiuqi.util.StringArrayOutputStream;
import java.util.Iterator;
import java.util.ArrayList;
import java.util.List;
import java.util.HashMap;
import java.util.Map;

public final class DBTrace
{
    private static Object syncObj;
    private static boolean doTrace;
    private static Map traceFinder;
    
    static {
        DBTrace.syncObj = new Object();
        DBTrace.doTrace = false;
        DBTrace.traceFinder = new HashMap();
    }
    
    private DBTrace() {
    }
    
    public static void createConnection(final Object key) {
        if (!DBTrace.doTrace) {
            return;
        }
        synchronized (DBTrace.syncObj) {
            final TraceEntry ex = new TraceEntry(key);
            DBTrace.traceFinder.put(key, ex);
            monitorexit(DBTrace.syncObj);
        }
    }
    
    public static void closeConnection(final Object key) {
        if (!DBTrace.doTrace) {
            return;
        }
        synchronized (DBTrace.syncObj) {
            DBTrace.traceFinder.remove(key);
            monitorexit(DBTrace.syncObj);
        }
    }
    
    public static void setTrace(final boolean value) {
        if (DBTrace.doTrace == value) {
            return;
        }
        if (value) {
            System.out.println("Strarted Trace DB Connection Objects");
            DBTrace.doTrace = true;
        }
        else {
            System.out.println("Stoped Trace DB Connection Objects");
            DBTrace.traceFinder.clear();
            DBTrace.doTrace = false;
        }
    }
    
    public static List getTraceResult() {
        synchronized (DBTrace.syncObj) {
            final List result = new ArrayList();
            final Iterator it = DBTrace.traceFinder.keySet().iterator();
            while (it.hasNext()) {
                final Object obj = DBTrace.traceFinder.get(it.next());
                if (obj != null) {
                    result.add(obj);
                }
            }
            final List list = result;
            monitorexit(DBTrace.syncObj);
            return list;
        }
    }
    
    public static List getTraceStrings() {
        final List traces = getTraceResult();
        final List result = new ArrayList();
        for (int i = 0; i < traces.size(); ++i) {
            final StringArrayOutputStream os = new StringArrayOutputStream();
            final PrintStream ps = new PrintStream((OutputStream)os);
            final TraceEntry item = traces.get(i);
            item.printStackTrace(ps);
            result.add(os.toStringArray());
        }
        return result;
    }
    
    public static boolean inTracing() {
        return DBTrace.doTrace;
    }
    
    public static void startLog(final int fileCount, final int fileSize, final String fileDir) {
        final PrintStream ps = new PrintStream(null, true);
        ps.flush();
    }
    
    public static final class TraceEntry extends Exception
    {
        private final Calendar createTime;
        private final String createThread;
        private static final int OBJ_UNKNOWN = 0;
        private static final int OBJ_CONNECTION = 1;
        private static final int OBJ_STATEMENT = 2;
        private static final int OBJ_RESULTSET = 3;
        private static final String[] OBJ_NAMES;
        private int objType;
        
        static {
            OBJ_NAMES = new String[] { "UNKNOWN", "CONNECTION", "STATEMENT", "RESULTSET" };
        }
        
        public TraceEntry(final Object obj) {
            this.objType = 0;
            this.createTime = Calendar.getInstance();
            this.createThread = Thread.currentThread().getName();
            if (obj instanceof Connection) {
                this.objType = 1;
            }
            else if (obj instanceof Statement) {
                this.objType = 2;
            }
            else if (obj instanceof ResultSet) {
                this.objType = 3;
            }
        }
        
        public Calendar getCreateTime() {
            return this.createTime;
        }
        
        public String getCreateThread() {
            return this.createThread;
        }
        
        @Override
        public String toString() {
            final DateFormat df = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss(SSS)");
            final StringBuffer buffer = new StringBuffer(super.toString());
            buffer.append(" (").append(TraceEntry.OBJ_NAMES[this.objType]).append(')').append(" created by [").append(this.createThread).append("], on ").append(df.format(this.createTime.getTime()));
            return buffer.toString();
        }
    }
}
